import java.util.*;
import java.awt.*;

/** Hauptprogramm, um Zeichdaten aus Files einzulesen,
  * und in einem Fenster graphisch wieder auszugeben.
  *
  * @author Bernhard Fiser 0109815
  */
public class Banner
{
	/* Hashmap zur Speicherung der Zeichendaten */
	static HashMap		hmap;
	/* Liste zur Speicherung der Bannertexte */
	static LinkedList	belem;

	/** Hauptprogramm */
	public static void main(String argv[])
	{
		char		c;
		CharDef		cd;
		CharElement	ce;
		ReadFile	f;
		String		line;
		int		i, e;
		StringTokenizer st;
		String		t;
		BannerElement	be;


		/* Falls nicht gengend Parameter bergeben wurden, 
		 * Fehler ausgeben und Programm beenden. */
		if (argv.length < 2)
		{
			System.err.println("*** usage: java Banner <data-directory> <textfile>");
			System.exit(1);
		}

		/* Hashmap initialisieren */
		hmap = new HashMap();
		/* Schleife ber alle Grobuchstaben von A bis Z */
		for (c = 'A'; c <= 'Z'; c++)
		{
			/* Information auf stdout ausgeben */
			System.out.print("info: reading character " + c + " ... ");
			
			/* Zeilenzhler initialisieren*/
			i = 0;
			/* Zeichendatenobjekt initialisieren mit entsprechendem
			 * Zeichen c */
			cd = new CharDef(c);
			/* File des Zeichens ffnen */
			f = new ReadFile(argv[0], c + ".def");
			/* Zeilen der Datei einzeln auslesen */
			while ((line = f.readLine()) != null)
			{
				/* Zeilenzhler erhhen */
				i++;
				/* Stringtokenizer initialisieren, Trennzeichen ' ' */
				st = new StringTokenizer(line, " ");
				/* Fehlercodevariable auf 0 setzen */
				e = 0;
				/* Falls zumindes ein Element existiert ... */
				if (st.hasMoreTokens())
				{
					/* ... erstes Element lesen */
					t = st.nextToken();
					/* Fehlercode je nach erstem Element auf einen Wert setzen */
					if (t.compareToIgnoreCase("strich") == 0)
						e = 1;
					else if (t.compareToIgnoreCase("kurve") == 0)
						e = 2;
					else
						e = -1;
				}

				ce = null;
				switch (e)
				{
					/* Falls Fehlercode e == 1, dann Linienelement initialisieren */
					case 1 :
						ce = new CharLineElement();
						break;

					/* Falls Fehlercode e == 2, dann Kurvenelement initialisieren */
					case 2 :
						ce = new CharCurveElement();
						break;

					/* Alle anderen Fehlercodes sind unzulssig, geben eine Fehlermeldung aus
					 * und beenden das Programm */
					default :
						System.err.println("*** error: syntax error in file " + c + ".def line " + i);
						System.exit(1);
				}
				
				/* aus Datei gelesener String in Zeichenelement bernehmen */
				if (!ce.parseLine(line))
				{
					/* Falls bernahme nicht mglich, dann Fehlermeldung ausgeben und
					 * Programm beenden */
					System.err.println("*** parse error: syntax error in file " + c + ".def line " + i);
					System.exit(1);
				}
				/* andernfalls dieses Zeichenelement dem Zeichen hinzufgen, d.h. alles OK */
				cd.add(ce);
			}
			/* Alle Zeilen des Zeichens gelesen, Datei schlieen */
			f.close();
			/* Zeichen in Hashmap aufnehmen */
			hmap.put(cd.getCharObject(), cd);
			/* Erfolgsmeldung auf Bildschirm ausgeben */
			System.out.println("done");
		}

		/* LinkedList erstellen, zur Ablage der Bannertexte */
		belem = new LinkedList();

		System.out.print("reading textfile " + argv[1] + " ... ");
		/* File ffnen und Zeilenzhler auf 0 setzen */
		i = 0;
		f = new ReadFile(argv[0], argv[1]);
		/* Zeilen einzeln auslesen */
		while ((line = f.readLine()) != null)
		{
			/* Zeilenzhler erhhen */
			i++;
			/* Neues BannerElement erstellen */
			be = new BannerElement();
			/* Gelesene Zeile parsen und in Bannerelement bernehmen */
			if (!be.parseLine(line))
			{
				/* Falls nicht erfolgreich Fehlermeldung ausgeben und Programm beenden */
				System.err.println("*** error: syntax error in file " + argv[1] + " line " + i);
				System.exit(1);
			}
			/* Alles OK, Bannerelement in Liste aufnehmen */
			belem.add(be);
		}
		/* Datei schlieen und Erfolgsmeldung ausgeben */
		f.close();
		System.out.println("done");

		/* Fenster ffnen und darstellen */
		new Wnd(hmap, belem);
		}
}
